function [a1,a2] = catman_read(%file)
// CATMAN_READ
// 
//               The input argument file is optional. It contains the
//               complete filename of the binary catman data file.
//               If the input argument is missing, the File Open GUI
//               appears in order to load files with extension BIN.
// 
//               ONLY CATMAN 4.5 and CATMAN 5.0 is supported !!!
//               The catman online data format is not supported.
// 
//               The result of reading can be found in the structured
//               variables a1 and a2.
// 
//               CATMAN is a software family for DAQ purposes. It is
//               optimized for DAQ with the amplifier systems MGCplus,
//               MGCsplit and Spider8 of the company HBM GmbH
//               (Hottinger Baldwin Messtechnik GmbH).
//               see also: http://www.hbm.com
//               Especially the software catman easy enables the fast and
//               easy measurement with HBM hardware like Spider8 and
//               MGCplus. Basis is TEDS based upon IEEE1451.4. Now you can
//               direct import catman datafiles into MatLab
// 
//               Fields in a1 (Content of Global Section):
//               -----------------------------------------
//               filename
//               fileid          -   Must be greater than 5010 !
//               dataoffset      -   there, the raw data is contained
//               comment         -   The channel comment itself
//               ReservedSTR     -   the reserved strings      (catman 5.0)
//               noofchan        -   Number of channels
//               mcl:            -   max. channel length  
//               redufact        -   reduction factor
// 
//               Fields in a2 (Channel Header section + Data area)
//               -------------------------------------------------
// 
//               Channelnumber   -   catman database channel
//               ChannelLength   -   no. of values in the DB channel
//               ChannelName     -   Name of the DB channel
//               Unit
//               comment
//               format          -   0=numeric, 1=String, 2=Binary Object
//               datawidth       -   numeric = 8, string >= 8
//               datumzeit       -   Date/Time of measurement
//               header          -   Traceability data, must be decoded
//               linmode         -   Linearization Mode
//               userscale
//               DBSensorInfo
//               data            -   The REAL content of the database
// 
//       Questions ?
//       Dr. Andreas Geissler, HBM GmbH  (http://www.hbm.com)
//       andreas.geissler@hbm.com 

// Ouput variables initialisation (not found in input variables)
a1=[];
a2=[];

// Number of arguments in function call
[%nargout,%nargin] = argn(0)

// Display mode
mode(0);

// Display warning for floating point exception
ieee(1);



a1 = [];
a2 = [];
if %nargout==0 then
  // Return, if there is no output argument
  return;
end;
if %nargin==0 then
  // !! L.64: Matlab function uigetfile not yet converted, original calling sequence used
  %file = tk_getfile('*.bin');
  if isempty(%file) then
    return;
  end;
end;
// !! L.71: No Scilab equivalent for: Matlab exist(), mtlb_exist() is called
// !! L.71: mtlb_exist() is just a partial emulation of Matlab exist() function !

fid = mtlb_fopen(%file,"rb");

// READING GLOBAL SECTION
// ----------------------

// L.81: No simple equivalent, so mtlb_fread() is called
fileid = mtlb_fread(fid,1,"short");
if fileid<5010 then
  disp("supported catman version: 4.5 or higher !");
  return;
end;
a1.filename = %file;
// File Identifier: Must be greater equal
a1.fileid = fileid;
// 5010 to proceed on.

// L.89: No simple equivalent, so mtlb_fread() is called
dataoffset = mtlb_fread(fid,1,"long");
// Offset in Byte from start of file for Data Area

// L.91: No simple equivalent, so mtlb_fread() is called
Lcomment = mtlb_fread(fid,1,"short");
// Length of file comment

// L.93: No simple equivalent, so mtlb_fread() is called
comment = mtlb_fread(fid,[1,Lcomment],"char");
a1.comment = ascii(comment);

if fileid==5010 then
  // L.97: No simple equivalent, so mtlb_fread() is called
  noofchan = mtlb_fread(fid,1,"short");
  // no. of channels
  a1.noofchan = noofchan;

  // L.100: No simple equivalent, so mtlb_fread() is called
  mcl = mtlb_fread(fid,1,"long");
  // max. channel length, normally=0,
  a1.mcl = mcl;
  // used for special append mode

  // L.103: No simple equivalent, so mtlb_fread() is called
  offsetchannel = mtlb_fread(fid,[1,noofchan],"long");
  //a1.offsetchannel=offsetchannel;

  // L.106: No simple equivalent, so mtlb_fread() is called
  redufact = mtlb_fread(fid,1,"long");
  // Reduction factor, normally =0,
  a1.redufact = redufact;
  // used for file compression
end;
if fileid>=5011 then
  // Read catman 5.0 format
  Lres = zeros(1,32);
  ReservedString = ascii(32 .*ones(32,256));

  for p = 1:1:32
    // L.114: No simple equivalent, so mtlb_fread() is called
    Lresakt = mtlb_fread(fid,1,"short");
    Lres(p) = Lresakt;
    // L.116: No simple equivalent, so mtlb_fread() is called
    restring = ascii(mtlb_fread(fid,[1,Lresakt],"char"));
    if max(size(asciimat(restring)))>256 then
      restring = part(restring,1:256);
      Lresakt = 256;
    end;
    //ReservedString(p,1:Lresakt) = restring;
  end;
  //a1.ReservedSTR = mtlb_setstr(ReservedString);

  // L.125: No simple equivalent, so mtlb_fread() is called
  noofchan = mtlb_fread(fid,1,"short");
  // no. of channels
  a1.noofchan = noofchan;

  // L.128: No simple equivalent, so mtlb_fread() is called
  mcl = mtlb_fread(fid,1,"long");
  // max. channel length, normally=0,
  a1.mcl = mcl;
  // used for special append mode

  // L.131: No simple equivalent, so mtlb_fread() is called
  offsetchannel = mtlb_fread(fid,[1,noofchan],"long");
  //a1.offsetchannel=offsetchannel;

  // L.134: No simple equivalent, so mtlb_fread() is called
  redufact = mtlb_fread(fid,1,"long");
  // Reduction factor, normally =0,
  a1.redufact = redufact;
  // used for file compression
end;

if %nargout==1 then
  return;
end;

// READ CHANNEL HEADER SECTION
// ---------------------------
CHINFO = [];
for p = 1:1:noofchan

  // L.147: No simple equivalent, so mtlb_fread() is called
  chaninfo.Channelnumber = mtlb_fread(fid,1,"short");
  // Channel location in catman database
  // L.148: No simple equivalent, so mtlb_fread() is called
  chaninfo.ChannelLength = mtlb_fread(fid,1,"long");
  // L.149: No simple equivalent, so mtlb_fread() is called
  LChanName = mtlb_fread(fid,1,"short");
  // Length of channel name
  if LChanName>0 then
    // L.151: No simple equivalent, so mtlb_fread() is called
    chaninfo.ChannelName = ascii(mtlb_fread(fid,[1,LChanName],"char"));
  else
    chaninfo.ChannelName = "";
  end;
  // L.155: No simple equivalent, so mtlb_fread() is called
  LunitName = mtlb_fread(fid,1,"short");
  // Unit of the channel
  if LunitName>0 then
    // L.157: No simple equivalent, so mtlb_fread() is called
    chaninfo.Unit = ascii(mtlb_fread(fid,[1,LunitName],"char"));
  else
    chaninfo.Unit = "";
  end;
  // L.161: No simple equivalent, so mtlb_fread() is called
  Lchcom = mtlb_fread(fid,1,"short");
  // channel comment
  if Lchcom>0 then
    // L.163: No simple equivalent, so mtlb_fread() is called
    chaninfo.comment = ascii(mtlb_fread(fid,[1,Lchcom],"char"));
  else
    chaninfo.comment = "";
  end;

  // L.168: No simple equivalent, so mtlb_fread() is called
  chaninfo.format = mtlb_fread(fid,1,"short");
  // Channel format:
  // 0=numeric, 1=String, 2=Binary Object
  // L.170: No simple equivalent, so mtlb_fread() is called
  chaninfo.datawidth = mtlb_fread(fid,1,"short");
  // numeric = 8, string >= 8
  // L.171: No simple equivalent, so mtlb_fread() is called
  chaninfo.datumzeit = mtlb_fread(fid,1,"double");
  // Date and Time of measurement
  // L.172: No simple equivalent, so mtlb_fread() is called
  extendedsize = mtlb_fread(fid,1,"long");
  // Size of extended channel header
  if extendedsize>0 then
    // L.174: No simple equivalent, so mtlb_fread() is called
    chaninfo.header = ascii(mtlb_fread(fid,[1,extendedsize],"char"));
  else
    chaninfo.header = "";
  end;
  // L.178: No simple equivalent, so mtlb_fread() is called
  chaninfo.linmode = mtlb_fread(fid,1,"char");
  // Linearization Mode
  // L.179: No simple equivalent, so mtlb_fread() is called
  chaninfo.userscale = mtlb_fread(fid,1,"char");
  // L.180: No simple equivalent, so mtlb_fread() is called
  npoi = mtlb_fread(fid,1,"char");
  // Number of points for user scale lin.
  // L.181: No simple equivalent, so mtlb_fread() is called
  mtlb_fread(fid,[1,npoi],"double");
  // The points
  // L.182: No simple equivalent, so mtlb_fread() is called
  mtlb_fread(fid,1,"short");
  // Thermo Type
  // L.183: No simple equivalent, so mtlb_fread() is called
  Lf = mtlb_fread(fid,1,"short");
  // Length of formula
  if Lf>0 then
    // L.185: No simple equivalent, so mtlb_fread() is called
    formula = ascii(mtlb_fread(fid,[1,Lf],"char"));
  end;
  if fileid>=5012 then
    // L.188: No simple equivalent, so mtlb_fread() is called
    SoDBinfo = mtlb_fread(fid,1,"long");  // Size of DBSensorInfo
    if SoDBinfo>0 then
      // L.190: No simple equivalent, so mtlb_fread() is called
      chaninfo.DBSensorInfo = mtlb_fread(fid,[1,SoDBinfo],"char");
    end;
  end;
  if isempty(CHINFO),
     CHINFO=chaninfo;
  else
    CHINFO = [CHINFO,chaninfo];
  end
end;

for p = 1:1:noofchan
  lakt = CHINFO(p).ChannelLength;
  if mtlb_logic(mtlb_double(lakt),">",0) then
    // L.200: No simple equivalent, so mtlb_fread() is called
    dataakt = mtlb_fread(fid,[1,lakt],"double");
  else
    dataakt = [];
  end;
  // ! L.204: mtlb(dataakt) can be replaced by dataakt() or dataakt whether dataakt is an M-file or not
  CHINFO(1,p).data = mtlb(dataakt);
end;

a2 = CHINFO;
mclose(fid);
endfunction
