function [filinf,CHINFO] = cp42cut(meafile,idx)
// cp42cut
//           Read MGCplus Data files, recorded with CP42
// 
//           [filinf,CHINFO]=cp42cut([meafile],[idx])
// 
//           meafile is the complete filename of the CP42 DAQ file.
//           idx is a 2 element vector containing the start- and end index
//           of samples to be read in.
// 
//           This function reads the data files recorded on a PC Card
//           harddisk on CP42. The input argument "meafile" is optional,
//           when it is missing, a user dialog for selecting a file with
//           extension *.me* is shown.
// 
//           filinf is a struct, containing the FileID, the number of
//           channels "ChannelC", Byte/Line, the data format, the data offset,
//           the time format, the file comment, the comment of the current
//           parameterset, the information about the creation date and the
//           number of used slots.
// 
//           CHINFO is a struct, containing information about the channel
//           number, the scaling, offset, unit, signalling bit, subchannel,
//           data, timeaxis, channel name and the channel code.
//           NEW:
//           Also the CHANNEL STATUS, including MGCplus LIMIT VALUES Switch
//           State, CHANNEL ERROR, OVERFLOW are contained in the field
//           status for each measurement channel
//               Bit 0: Status Limit Value 1
//               Bit 1: Status Limit Value 2
//               Bit 2: Status Limit Value 3
//               Bit 3: Status Limit Value  4
//               Bit 4: Overflow Gross Value
//               Bit 5: Overflow Net Value
//               Bit 6: calibration error
//               Bit 7: Change-Flag (amplifier scaling has changed)
// 
//           With PLOT(CHINFO(k).timeax,CHINFO(k).data) a measurement
//           signal can be displayed.
// 
//           There is a lot of information in the mea files about channels
//           and slots. Not all of these information is evaluated.
//           Want more ? Have a look at the word file "cp42cut-FORMAT.DOC"
// 
//           Only CP42 measurement files are supported. These DAQ files may
//           contain time information in following formats:
//           1) No Time Information
//           2) MGCplus Time (64 Bit)
//           3) NTP Time Format
//           The Time Formats "USB Frame Count" and 32 Bit MGC device Time
//           are not supported.
// 
//           IMPORTANT:
//           Only Measurement files with format 4 BYTE INT (Intel) are
//           supported.

filinf=[];
CHINFO=[];

// Number of arguments in function call
[%nargout,%nargin] = argn(0)

// Display mode
mode(0);

// Display warning for floating point exception
ieee(1);



CHINFO = [];
CHSET = [];
filinf = [];

if %nargin==0 then
  // No Input argument
  // !! L.61: Matlab function uigetfile not yet converted, original calling sequence used
  meafile = tk_getfile("*.me*");
  if isempty(meafile) then
    disp("File does not exist : "+meafile);
    return;
  end;
  idx=[-%inf,%inf];
end;

if %nargin==1 then
  // One input argument
  if type(meafile)==10 then     // string was given
    idx=[-%inf,%inf];
  else
    idx=meafile;
    meafile = tk_getfile("*.me*");
      if isempty(meafile) then
        disp("File does not exist : "+meafile);
        return;
      end
   end
end

if isempty(meafile) then
    disp("File does not exist : "+meafile);
    return;
end;

filinf = struct("FileID","ChannelC","ByteLine","MVLines","MWformat","MRate","DataOffset","TimeFormat","cmt","cmtpar","creation","usedslot","filename","filesize");
chaninfo = struct("Channelnumber","Scaling","Offset","Unit","SigBit","Subchannel","data","timeax","ChannelName","chncode","SigName","SigTypus");


chsetting = struct("ChannelName","chncode","ampcode","paraflg","tabquelle","acalflg","bridget","bridgev","uk","frequenz");

// Open CP42 measurement file (on PC) for reading.
fid = mtlb_fopen(meafile,"rb");
mseek(0,fid,"end");
// Evaluate the filesize in Byte
filesize = mtell(fid);
mseek(0,fid,"set");

// Read Header Block. The header block of all CP42 measurement files (also
// rainflow version CP42S30) is same: 8 long values
// L.92: No simple equivalent, so mtlb_fread() is called
a = mtlb_fread(fid,8,"long");

if a(1)~=6001 then
  // !! L.95: Matlab function code2str not yet converted, original calling sequence used
  disp("Invalid File identifier : "+code2str(a(1)));
  mclose(fid);
  return;
end;

if bool2s(a(8)==7001)|bool2s(a(8)==7003) then
  disp("32 Bit MGC Time and USB Frame Count are not supported");
  // !! L.102: Matlab function code2str not yet converted, original calling sequence used
  disp("Current Time Format Identifier : "+code2str(a(8)));
  mclose(fid);
  return;
end;

// Only 4 Byte INT Intel format is supported. With newest versions of the
// HBM setup assistant, only this format is upported.
if a(5)~=1253 then
  disp("Data not in 4 Byte INT (Intel) Format !");
  // !! L.111: Matlab function code2str not yet converted, original calling sequence used
  disp("Current Format Identifier : "+code2str(a(5)));
  mclose(fid);
  return;
end;

// The measurement rate is coded with values > 6000.
// Decoding to Hertz !
wrat = [6300:1:6320,6321:6341,6345:6347,6350:6361;1,2,5,10,25,50,60,75,100,150,200,300,400,600,800,1200,1600,2400,3200,4800,9600,3,4,6,8,15,20,30,48,64,80,96,120,160,192,240,320,384,480,640,960,1920,19200,38400,76800,1/2,1/5,1/10,1/20,1/50,1/100,1/200,1/500,1/1000,1/2000,1/5000,1/10000];
k0r = find(bool2s(wrat(1,:)==a(6)));
if isempty(k0r) then
  disp("unknown Measurement Rate : ");
  mclose(fid);
  return;
end;
SR = wrat(2,k0r);

noofsamp = floor((filesize-a(7)) ./a(3));
if a(4)~=noofsamp then
  // Sometimes some files do not have
  disp("Attention!");
  // info about no. of samples
  disp("Number of Samples contained");
  disp("in File seems to be invalid");
  a(4) = noofsamp;
end;
// (File Repairing)

idx=floor(idx);
idxmin=max(1,min(idx));
idxmax=min(a(4),max(idx));

filinf.FileID = a(1);
// File-ID (6001)
filinf.ChannelC = a(2);
// Channel Count (incl. Time Channels)
filinf.ByteLine = a(3);
// Size MeasVal-Line / Bytes
filinf.MVLines = a(4);
// No. of MeasVal-Lines
filinf.MWformat = a(5);
// MeasVal-Format
// (1253 is required: 4 Byte INT Intel)
filinf.MRate = SR;
// Sample Rate in Hz
filinf.DataOffset = a(7);
// Data Offset in Byte
filinf.TimeFormat = a(8);
// Time Channel Format
// (see TCS command, only with CP42)
filinf.filename = meafile;
filinf.filesize = filesize;

// Read Block with channel parameters
for ch = 1:a(2)
  chaninfo.Channelnumber = mtlb_fread(fid,1,"long");
  chaninfo.Scaling = mtlb_fread(fid,1,"float");
  chaninfo.Offset = mtlb_fread(fid,1,"float");
  chaninfo.Unit = ascii(mtlb_fread(fid,[1,4],"char"));
  chaninfo.SigBit = mtlb_fread(fid,1,"short");
  v01 = dec2bin(chaninfo.SigBit);
  sigbin = v01($:-1:1,:);
  // Convert Signal Bitfield
  // to binary representation
  if max(size(mtlb_double(sigbin)))>6 then // Only the lower 6 Bit
   sigbin = mtlb_e(sigbin,1:6); // contain sig. types!
  end;
  numsigs = mtlb_sum(mtlb_double(sigbin));
  // No. of Signals / Channels
  kitypus = mtlb_find(bool2s(mtlb_logic(mtlb_double(sigbin),"==",1)));
  // Positions of signals

  // L.170: No simple equivalent, so mtlb_fread() is called
  sucha = mtlb_fread(fid,1,"short");
  // Subchannel No.
  if sucha==0 then
    chaninfo.Subchannel = 1;
  else
    chaninfo.Subchannel = sucha;
  end;

  for sgtype = mtlb_imp(1,numsigs) // Evaluate Signal-
   typaktuell = kitypus(sgtype); // type information
  
   if typaktuell==1 then chaninfo.SigName = " (Gross)";end;
   if typaktuell==2 then chaninfo.SigName = " (Net)";end;
   if typaktuell==3 then chaninfo.SigName = " (PV1)";end;
   if typaktuell==4 then chaninfo.SigName = " (PV2)";end;
   if typaktuell==5 then chaninfo.SigName = " (CPV)";end;
   if typaktuell==6 then chaninfo.SigName = " (DIG)";end;
   chaninfo.SigTypus = typaktuell;
   if isempty(CHINFO),
      CHINFO=chaninfo;
   else
      CHINFO = [CHINFO,chaninfo];
  end
 end;
end;

// Read Block with amplifer settings
if a(8)==7000 then
  w0 = a(2)-2;  // w0 = Number of amplifiers used for DAQ
  mgctime = 1;  // mgctime=1, when recorded with MGC time
else
  w0 = a(2);
  mgctime = 0;
end;

for ch = 1:w0
  qtmp = ascii(mtlb_fread(fid,[1,47],"char"));
  // Read ChannelName
  // The channel name contains the name in hyphens and some stupid signs.
  // The channel names are redesigned in the following lines...
  ki34 = find(bool2s(abs(asciimat(qtmp))==34));
  // The channel name is between
  if max(size(ki34))>=2 then // two hyphens.
   qtmp = part(qtmp,ki34(1)+1:ki34(2)-1);
   qtmp = stripblanks(qtmp,1);
  end;
  ki59 = find(bool2s(abs(asciimat(qtmp))==59));
  // Sometimes, the strings contains
  if ~isempty(ki59) then // a final semicolon. It is removed
   v02 = ki59; qtmp = part(qtmp,mtlb_imp(1,mtlb_s(max(v02,firstnonsingleton(v02)),1)));
  end;
  qtmp = stripblanks(qtmp,1);
  // Remaining blanks are removed

  chsetting.ChannelName = qtmp;
  chsetting.chncode = mtlb_fread(fid,1,"char");
  chsetting.ampcode = mtlb_fread(fid,1,"char");
  chsetting.paraflg = mtlb_fread(fid,1,"char");
  chsetting.tabquelle = mtlb_fread(fid,1,"char");
  chsetting.acalflg = mtlb_fread(fid,1,"char");
  mseek(3,fid,"cur");
  chsetting.bridget = mtlb_fread(fid,1,"int16");
  chsetting.bridgev = mtlb_fread(fid,1,"char");
  statu = mseek(166,fid,"cur");
  sucha = mtlb_fread(fid,1,"char");
  if sucha==0 then
    chsetting.uk = 1;
  else
    chsetting.uk = sucha;
  end;
  if isempty(CHSET),
    CHSET=chsetting;
  else
    CHSET = [CHSET,chsetting];
  end
end;

for qz = 1:max(size(mtlb_double(CHSET)))
  // Append Chan. Name and Sig. type
  chanakt = CHSET(qz).chncode;
  sucha = CHSET(qz).uk;
  for qz2 = 1:max(size(mtlb_double(CHINFO)))
    chan2 = CHINFO(qz2).Channelnumber;
    sucha2 = CHINFO(qz2).Subchannel;
    if bool2s(mtlb_logic(mtlb_double(chanakt),"==",mtlb_double(chan2)))&bool2s(mtlb_logic(mtlb_double(sucha),"==",mtlb_double(sucha2))) then
      CHINFO(1,qz2).ChannelName = [CHSET(qz).ChannelName,CHINFO(qz2).SigName];
    end;
  end;
end;

// L.248: No simple equivalent, so mtlb_fread() is called
creation = ascii(mtlb_fread(fid,[1,26],"char"));
// Date and Time of File Creation
filinf.creation = creation;
// e. g. Wed Sep 03
// 14:43:06 1997

// L.252: No simple equivalent, so mtlb_fread() is called
comment = ascii(mtlb_fread(fid,[1,80],"char"));
// Comment of the measurement
filinf.cmt = comment;

// L.255: No simple equivalent, so mtlb_fread() is called
commentpar = ascii(mtlb_fread(fid,[1,80],"char"));
// Comment of active parameterset
filinf.cmtpar = commentpar;

// L.258: No simple equivalent, so mtlb_fread() is called
slotnumber = mtlb_fread(fid,1,"int");
// No. of used slots for meas.

nrow = max(size(CHINFO));
// If MGC Time Format (a(8)=7000),
// then two time channels are
// contained in CHINFO
if a(8)>7000 then
  nplus = 2;
else
  nplus = 0;
end;

mseek(a(7)+(nrow+nplus)*(idxmin-1)*4,fid,"set");
// Goto Data Section
data = mtlb_fread(fid,[nrow+nplus,(idxmax-idxmin+1)],"int32");
// Read Data of all chan. & all sig.
data = double(data);

status = data-256 .*floor(data ./256);
// Extract status information

for p = 1:nrow
  data(p,:) = mtlb_s((data(p,:) .*mtlb_double(CHINFO(p).Scaling)) ./1966080000,mtlb_double(CHINFO(p).Offset));
  CHINFO(1,p).data = data(p,:);
  // Append MV's to Channel Info
  CHINFO(1,p).status = status(p,:);
end;

// Build Time channel
if a(8)==7000 then
  // MGCplus Time Channel
  timeax = data(max(size(CHINFO)),:)+data(max(size(CHINFO))-1,:);
  // timeax = timeax-timeax(1);
else
  if a(8)==7002 then
    timeax = data(nrow+nplus-1,:)+data(nrow+nplus,:) ./1000000;
    // timeax = timeax-timeax(1);
  else
    timeax = (0:1:a(4)-1) ./SR;
  end;
end;

for p = 1:max(size(CHINFO))
  CHINFO(1,p).timeax = timeax;
end;

if mgctime then
  CHINFO = CHINFO(1:max(size(CHINFO))-2);
  // Only meas. channels !
  // Time information is
  // appended. CHINFO can
  // reduced by the last 2
  // rows
end;
mclose(fid);
// Close measurement file

// END OF cp42cut
endfunction
