function [rs,rsneu]=difrakti(d,h,option);
// DIFRAKTI	Difraction points
//
//		[RS,RSCOL]=DIFRAKTI(D,H,OPTION)
//
//		D		:	Vector with distances
//		H		:	Vector with height values
//		OPTION	:	Option vector, consisting of 5 values
//					1):	Irrelevance factor for heights		(0)
//					2):	Irrelevance mode [1,2,3]			(1)
//					3):	min. length of segments analyzed	(0)
//					4):	graphic display (0=no, 1=yes)		(1)
//					5):	Collection parameter				(30)
//
//		RS		:	Matrix with 2 columns. column 1 contains X-, column 2
//					contains Y vector of difraction points
//		RSCOL	:	collected difraction points

[%nargout,%nargin]=argn(0);

op0=[0,1,0,1,30];							// Voreinstellung Optionsvektor

if %nargin==1 then,								// Nur die Matrix DH wurde eingegeben
	DH=d;								    // 1. Zeile = Distanz
	d=DH(1,:); h=DH(2,:);					// 2. Zeile = Hhe
	option=op0;							    // Optionsvektor gem Voreinstellung
end
if %nargin==2 then,								// Distanz und Hhe wurden eingegeben
	option=op0;							    // Optionen gem Voreinstellung
end

DH=[d(:),h(:)].';							// Distanz, Hhe -> MATRIX mit zwei Zeilen
									        // Rekursive Berechnung von Haupthindernissen
resultat=difrarek(DH,option(1:3));			// Zeile 1: X-Wert, Zeile 2: Y-Wert,
									        // Zeile 3: eff. Eindringhhe

if isempty(resultat),						// LOS - PUR
	rs=[];
	rsneu=[];
end

if ~isempty(resultat),
	rtmp=resultat.';
	[a,ks]=gsort(rtmp(:,1));				// Beugungspunktmatrix nach Entfernung
	rtmp=rtmp(ks,:); 						// sortieren
	rtmp=rtmp($:-1:1,:);
	resultat=rtmp.';

	resu=resultat(1:2,:);					// Visualisierung des Beugungspfads
	resu=[DH(:,1),resu,DH(:,size(DH,2))];
else
	resu=[];
end
if option(4),
	if ~isempty(resu),
		plot2d(d,h,style=3);
		plot2d(resu(1,:),resu(2,:),style=2);
		plot2d(resultat(1,:),resultat(2,:),style=-2);
		xtitle('Profile','Distance','Height');
		// Die Matrix "RESU" enthlt neben den
		// Koordinaten der Beugungspunkte zustzlich
		// die Start- und Endpunktkoordinaten
	else
		plot2d(d,h,style=3);					// Grafik im Fall LOS
		xtitle('Profile (LOS)','Distance','Height');
	end
end
if isempty(resu),
	return;
end
dxdy=mtlb_diff(resu.');
dx=[dxdy(1:size(dxdy,1)-1,:),dxdy(2:size(dxdy,1),:)];
ds=sqrt( ((dx(:,1)+dx(:,3)) .^2) + ((dx(:,2)+dx(:,4)) .^2) );

rs=resultat.';
rs=[rs,dx,ds];

mindist=option(5)*mean(mtlb_diff(d));				// Gruppieren benachbarter Punkte

ki1=find(rs(:,4)>mindist);
if isempty(ki1),
	kv=[1;size(rs,1)];
else
	ki1=ki1(:).';
	if min(ki1)>1, ki1=[1,ki1]; end;
	ki2=[ki1(:,2:1:$),zeros(size(ki1,1),1)]
	//ki2=shiftmat(ki1,-1,0)-1;
	ki2($)=size(rs,1);
	kv=[ki1;ki2];
end

rsneu=[];
if size(kv,2)>=1,
	for k=1:1:size(kv,2),
		rsakt=rs(kv(1,k):kv(2,k),:);
		Z=size(rsakt,1);
		if Z==1, rsneu=[rsneu;rsakt(1:3),1]; end		// Einzelnes Hindernis
		if Z>1,
			dy=mtlb_diff(sign(rsakt(:,5)));
			if mtlb_any(dy<0),					            // Rounded Obstacle
				rsc=[rsakt(min(find(dy<0)),1:3),2];
				rsneu=[rsneu;rsc];
			else						                // Ausbreitung ber
				kx=find(rsakt(:,3)==max(rsakt(:,3)));	// Ebene
				if length(kx)>1, kx=kx(1); end
				rsneu=[rsneu;rsakt(kx,1:3),3];
			end
		end
	end
else
	rsneu=[];
end

rs=rs(:,1:2);
rsneu(:,3)=rsneu(:,4); rsneu=rsneu(:,1:3);

if option(4) & ~isempty(rsneu),
	plot2d(rsneu(:,1),rsneu(:,2),style=-7);
end

endfunction;

function resu=difrarek(DHin,option,kin,resu)
// DIFRAREK	Recursive calculation of path
//
//		RESU=DIFRAREK(DH,OPTION)
//
//		The Matrix DH contains in the 1st row the distances in
//		ascending order and in the 2nd row the terrain heights.
//
//		Vektor OPTION:
//		OPTION(1):	Irrelevance factor eff. Height {0,...0.2}
//		OPTION(2):	= 1 (1st eff. Height determines irrelevant height)
//					= 2 (height before determines irrelevant height)
//					= 3 (mean of heights before determine irrelevant height)
//		OPTION(3):	minimum length of a segment

[%nargout,%nargin]=argn(0);
if %nargin==1 then,						// Eingabe-Argument-berprfung
	option=[0.1,1,0];
	kin=1;
	resu=[];
end
if %nargin==2,
	kin=1;
	resu=[];
end

hf=option(1);
krit=option(2);
dkrit=option(3);

if (size(DHin,2)<=1) | (kin<=0) then,
	return;
end	// 1. + 2. Abruchbedingung

d=DHin(1,:); h=DHin(2,:);				        // Coordinate transformation
d=d - d(1); h=h - h(1);

if max(d)<=dkrit then,
	return;
end;

Dtmp=[d;h];
alp=atan(h(length(h)) ./d(length(d)));
A=[cos(alp),sin(alp);-sin(alp),cos(alp)];
DHneu=A*Dtmp;
dneu=DHneu(1,:); hneu=DHneu(2,:);

if hf~=0 & ~isempty(resu),				        // Kriterium: Letzte eff.
	[zresu,sresu]=size(resu);			        // positive Eindringhhe
	if krit==1 then, hohalt=resu(3,1); end
	if krit==2 then, hohalt=resu(3,sresu); end
	if krit==3 then, hohalt=mean(resu(3,:)); end
	hneu=hneu - (hf*hohalt) .*(hohalt>0);
end

if mtlb_any(hneu>0),
	kmax=[];
	d1=(dneu - dneu(1));
	d1(1)=1;
	d2=(dneu(length(dneu))- dneu);
	d2(length(d2))=1;
	if mtlb_any(d1<0) | mtlb_any(d2<0),
		if mtlb_any(d1<0),
			kmax=find(d1==min(d1) & (hneu>0));
		end
		if mtlb_any(d2<0),
			kmax=find(d2==min(d2) & (hneu>0));
		end
	end
	if ~(mtlb_any(d1<0) | mtlb_any(d2<0)) | isempty(kmax) then
		v=real(hneu .*sqrt((d1 + d2) ./(d1 .*d2)));
		v(1)=-0.1; v($)=-0.1;
		kmax=find(v==max(v) & (v>0) & (hneu>0));	// Haupthindernis
	end

	if length(kmax)>1 then,
		hmax=hneu(kmax);
		kmax=kmax(hmax==max(hmax));
		if length(kmax)>1 then,
			kmax=min(kmax);
		end
	end
	if length(kmax)==0, return; end				    // 3. Abbruchbedingung
	DH1=DHin(:,1:kmax);
	DH2=DHin(:,kmax:size(DHin,2));
	resuakt=[DHin(:,kmax);hneu(kmax)];
	resu=[resu,resuakt];
	resu=difrarek(DH1,option,kin+1,resu);			// rekursive Berechnung
	resu=difrarek((DH2),option,kin+1,resu);
else
	kout=-1;
	DHout=DHin;
	return;
end
endfunction;