function [x] = erfinv(y)
//ERFINV Inverse error function.
//   X = ERFINV(Y) is the inverse error function for each element of Y.
//   The inverse error function satisfies y = erf(x), for -1 <= y <= 1
//   and -inf <= x <= inf.
// 
//   See also ERF, ERFC, ERFCX, ERFCINV.

// Ouput variables initialisation (not found in input variables)
x=[];

// Display mode
mode(0);

// Display warning for floating point exception
ieee(1);

if ~isreal(mtlb_double(y),0) then
  error("Y must be real.");
end;
%v0 = size(mtlb_double(y));
x = zeros(%v0(1),%v0(2));

// Coefficients in rational approximations.
a = [0.886226899,-1.645349621,0.914624893,-0.140543331];
b = [-2.118377725,1.442710462,-0.329097515,0.012229801];
c = [-1.970840454,-1.624906493,3.429567803,1.641345311];
d = [3.5438892,1.6370678];

// Central range
y0 = 0.7;
k = mtlb_find(bool2s(mtlb_logic(abs(mtlb_double(y)),"<=",y0)));
if ~isempty(k) then
  z = mtlb_double(mtlb_e(y,k)) .*mtlb_double(mtlb_e(y,k));
  x = mtlb_i(x,k,(mtlb_double(mtlb_e(y,k)) .*mtlb_a(mtlb_a(mtlb_a(a(4)*z,a(3)) .*z,a(2)) .*z,a(1))) ./mtlb_a(mtlb_a(mtlb_a(mtlb_a(b(4)*z,b(3)) .*z,b(2)) .*z,b(1)) .*z,1));
end;

// Near end points of range
k = mtlb_find(bool2s(bool2s(mtlb_logic(y0,"<",mtlb_double(y)))&bool2s(mtlb_logic(mtlb_double(y),"<",1))));
if ~isempty(k) then
  z = sqrt(-log(mtlb_s(1,mtlb_double(mtlb_e(y,k)))/2));
  x = mtlb_i(x,k,mtlb_a(mtlb_a(mtlb_a(c(4)*z,c(3)) .*z,c(2)) .*z,c(1)) ./mtlb_a(mtlb_a(d(2)*z,d(1)) .*z,1));
end;

k = mtlb_find(bool2s(bool2s(mtlb_logic(-y0,">",mtlb_double(y)))&bool2s(mtlb_logic(mtlb_double(y),">",-1))));
if ~isempty(k) then
  z = sqrt(-log(mtlb_a(1,mtlb_double(mtlb_e(y,k)))/2));
  x = mtlb_i(x,k,-mtlb_a(mtlb_a(mtlb_a(c(4)*z,c(3)) .*z,c(2)) .*z,c(1)) ./mtlb_a(mtlb_a(d(2)*z,d(1)) .*z,1));
end;

// The relative error of the approximation has absolute value less
// than 8.9e-7.  One iteration of Halley's rational method (third
// order) gives full machine precision.

// Newton's method: new x = x - f/f'
// Halley's method: new x = x - 1/(f'/f - (f"/f')/2)
// This function: f = erf(x) - y, f' = 2/sqrt(pi)*exp(-x^2), f" = -2*x*f'

// Newton's correction
u = mtlb_s(erf(x),mtlb_double(y)) ./((2/sqrt(%pi))*exp(-x .^2));

// Halley's step
x = mtlb_s(x,u ./mtlb_a(1,x .*u));

// Exceptional cases
x = mtlb_i(x,mtlb_logic(mtlb_double(y),"==",-1),-%inf);
x = mtlb_i(x,mtlb_logic(mtlb_double(y),"==",1),%inf);
x = mtlb_i(x,mtlb_logic(abs(mtlb_double(y)),">",1),%nan);
x = mtlb_i(x,isnan(mtlb_double(y)),%nan);
endfunction
