function [fileinf,CHINFO] = meacomp(meafile,compfactor) 
// MEACOMP
//           [fileinf,CHINFO]=MEACOMP([MEAFILE],[COMPRESSION])
// 
//           MEAFILE:        Name of a CP42 measurement file with
//                           Measurement value format 4 Byte INT.
//           COMPRESSION:    Samples / Block. Default value=500
// 
//           MEACOMP reads the values of the CP42 measurement file.
//           A block containing all channels/signals is read. The
//           number of samples is specified by the variable
//           compression. From every channel/signal the max. and
//           min. value are subtracted, so that you get an information
//           about "signal activity" in each block.
// 
//           The information about the file is stored in the struct
//           variable fileinf, the information about channels and
//           signals is contained in the variable CHINFO.
//           CHINFO(k).data contains the "compressed" activity information,
//           CHINFO(k).timeax contains the sample index.

// Ouput variables initialisation (not found in input variables)
fileinf=[];
CHINFO=[];

// Number of arguments in function call
[%nargout,%nargin] = argn(0)

// Display mode
mode(0);

// Display warning for floating point exception
ieee(1);

CHINFO = [];
CHSET = [];
fileinf = [];
warning("off");

if %nargin==0 then
  // No Input argument
  // !! L.29: Matlab function uigetfile not yet converted, original calling sequence used
  meafile = tk_getfile("*.me*");
  if isempty(meafile) then
    disp("File does not exist : "+meafile);
    return;
  end;
  compfactor = 500;
end;

if %nargin==1 then
  // One Input argument
  if ~type(meafile)==10 then // If Graphic Option was specified...
   compfactor = meafile;
   meafile = tk_getfile("*.me*");
   if isempty(meafile) then
     disp("File does not exist : "+meafile);
     return;
  end;
  else // CP42 filename was specified
   compfactor = 500;
  end;
end;

if isempty(meafile) then
  disp("File does not exist : "+meafile);
  return;
end;

fileinf = struct("FileID","ChannelC","ByteLine","MVLines","MWformat","MRate","DataOffset","TimeFormat","cmt","cmtpar","creation","usedslot","filename","filesize");


chaninfo = struct("Channelnumber","Scaling","Offset","Unit","SigBit","Subchannel","data","timeax","ChannelName","chncode","SigName","SigTypus");


chsetting = struct("ChannelName","chncode","ampcode","paraflg","tabquelle","acalflg","bridget","bridgev","uk","frequenz");

// Open CP42 measurement file (on PC) for reading.
fid = mtlb_fopen(meafile,"rb");
mseek(0,fid,"end");
filesize = mtell(fid);   // Evaluate the filesize in Byte
mseek(0,fid,"set");

// Read Header Block. The header block of all CP42 measurement files (also
// rainflow version CP42S30) is same: 8 long values
a = mtlb_fread(fid,8,"long");

if a(1)~=6001 then
  disp("Invalid File identifier : "+int2str(a(1)));
  mclose(fid);
  return;
end;

if bool2s(a(8)==7001)|bool2s(a(8)==7003) then
   disp("32 Bit MGC Time and USB Frame Count are not supported");
   disp("Current Time Format Identifier : "+int2str(a(8)));
   mclose(fid);
   return;
end;

// Only 4 Byte INT Intel format is supported. With newest versions of the
// HBM setup assistant, only this format is upported.
if a(5)~=1253 then
  disp("Data not in 4 Byte INT (Intel) Format !");
  disp("Current Format Identifier : "+int2str(a(5)));
  mclose(fid);
  return;
end;

// The measurement rate is coded with values > 6000.
// Decoding to Hertz !



wrat = [6300:1:6320,6321:6341,6345:6347,6350:6361;1,2,5,10,25,50,60,75,100,150,200,300,400,600,800,1200,1600,2400,3200,4800,9600,3,4,6,8,15,20,30,48,64,80,96,120,160,192,240,320,384,480,640,960,1920,19200,38400,76800,1/2,1/5,1/10,1/20,1/50,1/100,1/200,1/500,1/1000,1/2000,1/5000,1/10000];
k0r = find(bool2s(wrat(1,:)==a(6)));
if isempty(k0r) then
  disp("unknown Measurement Rate : "+int2str(a(6)));
  mclose(fid);
  return;
end;
SR = wrat(2,k0r);

noofsamp = floor((filesize-a(7)) ./a(3));
if a(4)~=noofsamp then
  disp("Attention!");                    // Sometimes some files do not have
  disp("Number of Samples contained");   // info about no. of samples
  disp("in File seems to be invalid");
  disp("File : "+int2str(a(4)));
  disp("calc : "+int2str(noofsamp));
  a(1,4) = noofsamp;
end;
// (File Repairing)


fileinf.FileID = a(1);           // File-ID (6001)
fileinf.ChannelC = a(2);         // Channel Count (incl. Time Channels)
fileinf.ByteLine = a(3);         // Size MeasVal-Line / Bytes
fileinf.MVLines = a(4);          // No. of MeasVal-Lines
fileinf.MWformat = a(5);         // MeasVal-Format
                                   // (1253 is required: 4 Byte INT Intel)
fileinf.MRate = SR;              // Sample Rate in Hz
fileinf.DataOffset = a(7);       // Data Offset in Byte
fileinf.TimeFormat = a(8);       // Time Channel Format
                                   // (see TCS command, only with CP42)
fileinf.filename = meafile;
fileinf.filesize = filesize;

// Read Block with channel parameters
for ch = 1:a(2)
  chaninfo.Channelnumber = mtlb_fread(fid,1,"long");   // Channelnumber
  chaninfo.Scaling = mtlb_fread(fid,1,"float");        // Scale Factor
  chaninfo.Offset = mtlb_fread(fid,1,"float");         // Offset
  chaninfo.Unit = ascii(mtlb_fread(fid,[1,4],"char")); // Unit
  chaninfo.SigBit = mtlb_fread(fid,1,"short");         // Signal-Bitfeld

  v01_1 = dec2bin(chaninfo.SigBit);
  sigbin = v01_1($:-1:1,:);
  // Convert Signal Bitfield to binary representation
  if max(size(mtlb_double(sigbin)))>6 then     // Only the lower 6 Bit
   sigbin = mtlb_e(sigbin,1:6);                // contain sig. types!
  end;
  numsigs = mtlb_sum(mtlb_double(sigbin));     // No. of Signals / Channels
  kitypus = mtlb_find(bool2s(mtlb_logic(mtlb_double(sigbin),"==",1)));
                                               // Positions of signals

  sucha = mtlb_fread(fid,1,"short");
  // Subchannel No.
  if sucha==0 then
    chaninfo.Subchannel = 1;
  else
    chaninfo.Subchannel = sucha;
  end;

  for sgtype = mtlb_imp(1,numsigs)            // Evaluate Signal-
   typaktuell = kitypus(sgtype);              // type information
   if typaktuell==1 then chaninfo.SigName = " (Gross)";end;
   if typaktuell==2 then chaninfo.SigName = " (Net)";end;
   if typaktuell==3 then chaninfo.SigName = " (PV1)";end;
   if typaktuell==4 then chaninfo.SigName = " (PV2)";end;
   if typaktuell==5 then chaninfo.SigName = " (CPV)";end;
   if typaktuell==6 then chaninfo.SigName = " (DIG)";end;
   chaninfo.SigTypus = typaktuell;
   if isempty(CHINFO),
      CHINFO=chaninfo;
   else
      CHINFO = [CHINFO,chaninfo];
   end
  end;
end;

// Read Block with amplifer settings
if a(8)==7000 then
  w0 = a(2)-2;  // w0 = Number of amplifiers used for DAQ
  mgctime = 1;  // mgctime=1, when recorded with MGC time
else
  w0 = a(2);
  mgctime = 0;
end;

for ch = 1:w0
  qtmp = ascii(mtlb_fread(fid,[1,47],"char"));
  // Read ChannelName
  // The channel name contains the name in hyphens and some stupid signs.
  // The channel names are redesigned in the following lines...
  ki34 = find(bool2s(abs(asciimat(qtmp))==34));
  // The channel name is between
  if max(size(ki34))>=2 then // two hyphens.
   qtmp = part(qtmp,ki34(1)+1:ki34(2)-1);
   qtmp = stripblanks(qtmp,1);
  end;
  ki59 = find(bool2s(abs(asciimat(qtmp))==59));
  // Sometimes, the strings contains a final semicolon. It is removed
  if ~isempty(ki59) then
   v02_1 = ki59; qtmp = part(qtmp,mtlb_imp(1,mtlb_s(max(v02_1,firstnonsingleton(v02_1)),1)));
  end;
  
  qtmp = stripblanks(qtmp,1);                     // Remaining blanks are removed
  chsetting.ChannelName = qtmp;
  chsetting.chncode = mtlb_fread(fid,1,"char");   // Read Channel Code
  chsetting.ampcode = mtlb_fread(fid,1,"char");   // Read amplifier code
  chsetting.paraflg = mtlb_fread(fid,1,"char");   // parflag (1..8: intern, 9: XM001)
  chsetting.tabquelle = mtlb_fread(fid,1,"char"); // Where is the chart from ?
  chsetting.acalflg = mtlb_fread(fid,1,"char");   // AutoCalibration on / off ?
  mseek(3,fid,"cur");                             // Ignore loca_, meas_ and stat_flg
  
  chsetting.bridget = mtlb_fread(fid,1,"int16");  // Read Bridgetype
  chsetting.bridgev = mtlb_fread(fid,1,"char");   // Read Bridge supply voltage
  statu = mseek(166,fid,"cur");
  sucha = mtlb_fread(fid,1,"char");
  if sucha==0 then
    chsetting.uk = 1;
  else
    chsetting.uk = sucha;
  end;
  if isempty(CHSET),
    CHSET=chsetting;
  else
    CHSET = [CHSET,chsetting];
  end
end;

for qz = 1:max(size(mtlb_double(CHSET)))
  // Append Chan. Name and Sig. type
  chanakt = CHSET(qz).chncode;
  sucha = CHSET(qz).uk;
  for qz2 = 1:max(size(mtlb_double(CHINFO)))
    chan2 = CHINFO(qz2).Channelnumber;
    sucha2 = CHINFO(qz2).Subchannel;
    if bool2s(mtlb_logic(mtlb_double(chanakt),"==",mtlb_double(chan2)))&bool2s(mtlb_logic(mtlb_double(sucha),"==",mtlb_double(sucha2))) then
      CHINFO(1,qz2).ChannelName = [CHSET(qz).ChannelName,CHINFO(qz2).SigName];
    end;
  end;
end;

creation = ascii(mtlb_fread(fid,[1,26],"char"));
fileinf.creation = creation;                     // Date and Time of File Creation
                                                   // e. g. Wed Sep 03
                                                   // 14:43:06 1997

comment = ascii(mtlb_fread(fid,[1,80],"char"));    // Comment of the measurement
fileinf.cmt = comment;

commentpar = ascii(mtlb_fread(fid,[1,80],"char")); // Comment of active parameterset
fileinf.cmtpar = commentpar;
slotnumber = mtlb_fread(fid,1,"int");              // No. of used slots for meas.

mseek(a(7),fid,"set");                             // Goto Data Section

nrow = max(size(CHINFO));
// If MGC Time Format (a(8)=7000), then two time channels are contained in CHINFO
if a(8)>7000 then
  nplus = 2;
else
  nplus = 0;
end;

noblock = floor(a(4)/mtlb_double(compfactor));
restsamp = mtlb_s(a(4),noblock*mtlb_double(compfactor));
Bmin = 40;
if mtlb_logic(noblock,"<",Bmin) then
  compfactor = ceil(a(4) ./Bmin);
  noblock = floor(a(4)/mtlb_double(compfactor));
  restsamp = mtlb_s(a(4),noblock*mtlb_double(compfactor));
end;

anzneu=0;
anzold=0;

// Show a progression bar
winID=progressionbar('Done: '+int2str(anzneu)+' %');

data = [];
timeax = [];
for qz = mtlb_imp(1,1,noblock)
  data0 = mtlb_fread(fid,[nrow+nplus,compfactor],"int32");    // Read Data
  data0 = double(data0);
  for p = 1:nrow
    data0(p,:) = mtlb_s((data0(p,:) .*mtlb_double(CHINFO(p).Scaling)) ./1966080000,mtlb_double(CHINFO(p).Offset));
  end;
  v01_1 = data0.';
  v11_1 = data0.';
  exceed=(max(v01_1,'r')-min(v01_1,'r')).';
  //exceed = mtlb_s(mtlb_max(v01_1,firstnonsingleton(v01_1)),mtlb_min(v11_1,firstnonsingleton(v11_1))).';
  if isempty(data) then
    data=exceed;
  else
    data = [data,exceed];
  end
  idxstamp = compfactor*qz-250;
  if isempty(timeax) then
    timeax=idxstamp;
  else  
     timeax = [timeax,idxstamp];
  end
  anzneu = round((100*qz)/noblock);
  
  if anzneu>anzold+2;
      messge='Done: '+int2str(anzneu)+' %';
      progressionbar(winID, messge);
      anzold=anzneu;
  end
end;

if mtlb_logic(restsamp,">",0) then
  data0 = mtlb_fread(fid,[nrow+nplus,restsamp],"int32");   // Read Rest of Data
  data0 = double(data0);
  for p = 1:nrow
    data0(p,:) = mtlb_s((data0(p,:) .*mtlb_double(CHINFO(p).Scaling)) ./1966080000,mtlb_double(CHINFO(p).Offset));
  end;
  v01_1 = data0.';
  exceed=(max(v01_1,'r')-min(v01_1,'r')).';
  //exceed = mtlb_s(mtlb_max(v01_1,firstnonsingleton(v01_1)),mtlb_min(v11_1,firstnonsingleton(v11_1))).';
  if isempty(data) then
    data=exceed;
  else
    data = [data,exceed];
  end
  idxstamp = noblock*compfactor+(restsamp/2);
  if isempty(timeax) then
    timeax=idxstamp;
  else  
     timeax = [timeax,idxstamp];
  end
end;
winclose(winID);

for p = 1:max(size(CHINFO))
  datakt = data(p,:);
  v01_1 = datakt;
  umax = max(v01_1);
  umin = min(v01_1);
  if umax==umin then
    activity = zeros(1,length(datakt));
  else  
    activity = 100 .*(datakt - umin) ./(umax-umin);
  end
  CHINFO(1,p).data = activity;
  CHINFO(1,p).timeax = timeax;
  CHINFO(1,p).Unit = "Activity [%]";
end;

if mgctime then
  CHINFO = CHINFO(1:max(size(CHINFO))-2);
  // Only meas. channels !
  // Time information is appended. CHINFO can
  // reduced by the last 2 rows
end;
mclose(fid);                     // Close measurement file

// END OF MEACOMP
endfunction
