function [xx,yy,zz] = meshgrid(x,y,z)
//MESHGRID Generation of X and Y arrays for 3-D plots.
//[X,Y] = MESHGRID(x,y) transforms the domain specified by vectors
//x and y into arrays X and Y that can be used for the evaluation
//of functions of two variables and 3-D surface plots.
//The rows of the output array X are copies of the vector x and
//the columns of the output array Y are copies of the vector y.
// 
//For example, to evaluate the function  x*exp(-x^2-y^2) over the 
//range  -2 < x < 2,  -2 < y < 2,
// 
//[X,Y] = meshgrid(-2:.2:2, -2:.2:2);
//Z = X .* exp(-X.^2 - Y.^2);
//mesh(Z)
// 
//[X,Y] = MESHGRID(x) is an abbreviation for [X,Y] = MESHGRID(x,x).
// 
//[X,Y,Z] = MESHGRID(x,y,z) produces packed 3-D arrays that can
//be used to evaluate functions of three variables and 3-D
//volumetric plots.

// Ouput variables initialisation (not found in input variables)
xx=[];
yy=[];
zz=[];

// Number of arguments in function call
[%nargout,%nargin] = argn(0)

// Display mode
mode(0);

// Display warning for floating point exception
ieee(1);

if %nargin==1 then
  y = x;
end;
if bool2s(isempty(x))|bool2s(isempty(y)) then
  xx = [];
  yy = [];
  zz = [];
end;

xx = mtlb_0(mtlb_full(mtlb_e(x,:)));
// Make sure x is a full row vector.
yy = mtlb_full(mtlb_e(y,:));
// Make sure y is a full column vector.
nx = max(size(mtlb_double(xx)));
ny = max(size(mtlb_double(yy)));
xx = xx(ones(ny,1),:);
yy = yy(:,ones(1,nx));

if %nargin==3 then
  // 3-D array case
  nz = max(size(mtlb_double(z)));
  if nz==0 then
    xx = [];  yy = [];  zz = [];
    return;
  end;
  zz = mtlb_0(mtlb_full(mtlb_e(z,:)));
  // make sure z is a full row vector
  zz = zz(ones(nx*ny,1),:);
  xx = mtlb_e(xx,:);
  xx = xx(:,ones(1,nz));
  yy = mtlb_e(yy,:);
  yy = yy(:,ones(1,nz));
end;
endfunction
