function [Q,VS,zlist]=nextdrei(tri);
// NEXTDREI	Neighbour analysis of triangles and voronoy diagram
//
//		Q=NEXTDREI(TRI)					or
//		[Q,VOROLINES,DELAUNAYLINES]=NEXTDREI(P)
//
//		Variables:
//		TRI	     :	Indexmatrix of Delaunay triangle points
//		Q	     :	neighbour matrix - each column of Q represents
//				    a triangle, which is specified by the indexmatrix TRI
//		VOROLINES:	Matrix with 2 rows. Each column represents a voronoy line.
//		DELAUNAYLINES:	Matrix with 4 rows. The last row is identical to the
//				        1st row. Each column describes a delaunay triangle.
//		P	     :	complex Vector, contains points in R.
//
//
//		If only the index matrix of delaunay points is given, then you get the
//		neighbourmatrix of the delaunay triangles. If you use complex points as
//		input argument, and you have less than two output arguments, then you
//		will see also a graphical display of delaunay lines and voronoy diagram.

[%nargout,%nargin]=argn(0);

flaggy=0;
if ~isreal(tri),					// complex vector used as input			
	p=tri;						    // =>delaunay
	[mp,r,zlist,tri]=delaunay(p);	// Statusflag "FLAGGY" is set to 1
	flaggy=1;
end

tri=mtlb_sort(tri.').';				// Sorting the index matrix. The sequence
Q=[];							    // of triangles will not be changed.

P=meshgrid(1:1:size(tri,1),1:1:size(tri,1)).';
if flaggy==1,
	MP=meshgrid(mp,1:1:size(tri,1)).';
end

for k=1:1:size(tri,1),				// neighbour analysis of index matrix TRI
	triakt=tri(k,:);				// neighboured triangles have a common edge
	triakt=meshgrid(triakt,1:1:size(tri,1));
	Dv=zeros(1,size(tri,1));		// Therefore, 2 values in the corresp. rows
	for k=1:1:3,					// of TRI must be equal
		triakt=spinup(triakt.',1).';
							        // all possible sequences of values must
		D=abs(tri-triakt).';		// be taken into account.
		D=bool2s(D==0);				// Therefore use of "SPINUP"
		D=sum(D,'r');
		D=bool2s(D>=2);
		Dv=bool2s(Dv | D);
	end
	triakt=triakt(:,$:-1:1);
	for k=1:1:3,
		triakt=spinup(triakt.',1).';
		D=abs(tri-triakt).';
		D=bool2s(D==0);
		D=sum(bool2s(D),'r');
		D=bool2s(D>=2);
		Dv=bool2s(Dv | D);
	end
	Q=[Q,Dv(:)];			// completion of neighbour matrix Q.
end

// if complex points have been used as input argument,
// the cration of the voronoy diagram is done now

if flaggy==1 then, Qorg=Q; end	// List with neighbours
nunit=size(tri,1);
Q=(Q .*P) .*(bool2s(~eye(nunit,nunit)));
if flaggy==1 then,
	NBH=sum(bool2s(Q>0),'r');				// number of triangle neighbours
	INHU=ipolygon(p,mp).';
	MPBIN1=MP .*bool2s(Q>0);				// Start point of Voronoi edge
	MPBIN2=(MP.') .*bool2s(Q>0);			// End point of Voronoi edge
	VS=[MPBIN1(:),MPBIN2(:)].';				// connecting Voronoi points

	kselct2=find((NBH==2));			// each voronoy point must have 3 edges
	kselct1=find((NBH==1));			// =>Completion of edges

	mp2=mp(kselct2);				// Voronoi points with 2 edges
	mp1=mp(kselct1);				// Voronoi points with 1 edge

	pr=p(:).';
	pr=[real(pr);imag(pr)];
	[npoi,Vidx]=convex_hull(pr);
	V=p(Vidx);
	V=[V,V(1)];

	vcm=V(1:1:$-1)+0.5 .*(V(2:1:$)-V(1:1:$-1));

	VREST=[];
	if ~isempty(kselct1),
							// completion of edges:
							// triangles with one neighbour
		zpoi1=zlist(1:3,kselct1);
		inhu1=INHU(kselct1);
		for k=1:1:size(zpoi1,2),
			zpakt=zpoi1(:,k);		// current triangle
			[ZP,VM]=meshgrid(zpakt,V);
			D=abs(ZP - VM).';
			[kz,ks]=find(D==0);
			Vsel=V(ks); LVSEL=length(Vsel);

							// cutting points of middle vertices

			Vnewpoi=Vsel(1:1:$-1) + 0.5 .*(Vsel(2:1:$)-Vsel(1:1:$-1));
			
			if inhu1(k)==0,
				dztarget=Vnewpoi - mp1(k);
				target=mp1(k) + 0.99 .*dztarget;
				isitin=ipolygon(p,target);
				Vnewpoi(~isitin)=mp1(k) -0.8 .*dztarget(~isitin);
				LVSEL=length(Vnewpoi)+1;
			else
				kins=zeros(1,length(Vnewpoi));
				for l=1:1:length(Vnewpoi),
					d2pt=abs(Vnewpoi(l)-vcm);
					d2pt=min(d2pt);
					if d2pt==0,
						kins(l)=1;
					end
				end
				Vnewpoi=Vnewpoi(kins==1);
				LVSEL=length(Vnewpoi)+1;
			end
			if ~isempty(Vnewpoi),
			VSNEW=[(mp1(k) .*ones(length(Vnewpoi),1)),Vnewpoi(:)].';
			VREST=[VREST,VSNEW];
			end
		end
	end
	if ~isempty(kselct2),
							// completion of edges:
							// triangles with 2 neighbours
		zpoi2=zlist(1:3,kselct2);
		inhu2=INHU(kselct2);
		for k=1:1:size(zpoi2,2),
			zpakt=zpoi2(:,k);	// current triangle
			[ZP,VM]=meshgrid(zpakt,V);
			D=abs(ZP - VM).';
			[kz,ks]=find(D==0);
			Vsel=V(ks);
			LVSEL=length(Vsel);
			Vnewpoi=Vsel(1:1:$-1) + 0.5 .*(Vsel(2:1:$)-Vsel(1:1:$-1));
			if inhu2(k)==0,
				dztarget=Vnewpoi - mp2(k);
				target=mp2(k) + 0.99 .*dztarget;
				isitin=ipolygon(p,target);
				Vnewpoi(~isitin)=mp2(k) -0.8 .*dztarget(~isitin);
				LVSEL=length(Vnewpoi)+1;
			else
				kins=zeros(1,length(Vnewpoi));
				for l=1:1:length(Vnewpoi),
					d2pt=abs(Vnewpoi(l)-vcm);
					d2pt=min(d2pt);
					if d2pt==0 then,
						kins(l)=1;
					end
				end
				Vnewpoi=Vnewpoi(kins==1);
				LVSEL=length(Vnewpoi)+1;
			end
			if ~isempty(Vnewpoi) then,
				VSNEW=[(mp2(k) .*ones(length(Vnewpoi),1)),Vnewpoi(:)].';
				VREST=[VREST,VSNEW];
			end
		end
	end
	VS=[VS,VREST];
	dvs=abs(VS(1,:)-VS(2,:));
	VS=VS(:,dvs~=0);
end
if %nargout<=1 & flaggy==1 then
	plot2d(real(VS),imag(VS),style=2*ones(1,size(VS,2)));
	plot2d(real(zlist),imag(zlist),style=4*ones(1,size(zlist,2)));
	plot2d(real(p),imag(p),style=-3*ones(1,length(p)));
	f=get("current_figure");
	f.figure_name='Voronoy / Delaunay';
end
endfunction